package eu.dnetlib.validatorapi.controllers;

import eu.dnetlib.validatorapi.entities.SummaryResult;
import eu.dnetlib.validatorapi.entities.ValidationIssue;
import eu.dnetlib.validatorapi.repositories.ValidationIssueRepository;
import eu.dnetlib.validatorapi.repositories.ValidationJobRepository;
import eu.dnetlib.validatorapi.repositories.ValidationResultRepository;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@CrossOrigin(origins = "*")
@RequestMapping("/reports")
public class ReportController {
    private final ValidationJobRepository validationJobRepository;
    private final ValidationResultRepository validationResultRepository;
    private final ValidationIssueRepository validationIssueRepository;

    public ReportController(ValidationJobRepository validationJobRepository,
                            ValidationResultRepository validationResultRepository,
                            ValidationIssueRepository validationIssueRepository) {
            this.validationJobRepository = validationJobRepository;
            this.validationResultRepository = validationResultRepository;
            this.validationIssueRepository = validationIssueRepository;
        }


    @RequestMapping(value = {"getResultsByJobId"}, method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public List<SummaryResult> getJobResults(@RequestParam(name = "jobId") int jobId){
        return validationResultRepository.getSummaryResult(jobId);
    }

    @RequestMapping(value = {"getRecordsByRule"}, method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public List<ValidationIssue> getRecordsByRule(@RequestParam(name = "jobId") int jobId,
                                                  @RequestParam(name = "ruleName") String ruleName){
        return  validationIssueRepository.getErrors(jobId, ruleName);
    }

    @RequestMapping(value = {"getErrorsReport"}, method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public Map<String, List<String>> getErrors(@RequestParam(name = "jobId") int jobId){
        List<Object[]> resultList = validationIssueRepository.getAllErrorsRecordUrls(jobId);
        return extractRecordsGroupedByRule(resultList);
    }

    @RequestMapping(value = {"getWarningsReport"}, method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public Map<String, List<String>> getWarningsReport(@RequestParam(name = "jobId") int jobId,
                                                       @RequestParam(name = "ruleName") String ruleName){
        List<Object[]> resultList = validationIssueRepository.getAllWarningsRecordUrls(jobId);
        return extractRecordsGroupedByRule(resultList);
    }

    private Map<String, List<String>> extractRecordsGroupedByRule(List<Object[]> resultList) {
        Map<String, List<String>> recordUrlsByIssueText = new HashMap<>();

        for (Object[] result : resultList) {
            String issueText = (String) result[0];
            String recordUrl = (String) result[1];

            List<String> recordUrls = recordUrlsByIssueText.get(issueText);
            if (recordUrls == null) {
                recordUrls = new ArrayList<>();
                recordUrlsByIssueText.put(issueText, recordUrls);
            }
            recordUrls.add(recordUrl);
        }
        return recordUrlsByIssueText;
    }

}
