package eu.dnetlib.validatorapi.processors;

import eu.dnetlib.validator2.validation.XMLApplicationProfile;
import eu.dnetlib.validator2.validation.guideline.Guideline;
import eu.dnetlib.validator2.validation.guideline.openaire.AbstractOpenAireProfile;
import eu.dnetlib.validatorapi.entities.ValidationIssue;
import eu.dnetlib.validatorapi.entities.ValidationJob;
import eu.dnetlib.validatorapi.entities.ValidationRuleResult;
import eu.dnetlib.validatorapi.repositories.ValidationIssueRepository;
import eu.dnetlib.validatorapi.repositories.ValidationResultRepository;
import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.StringReader;
import java.util.Map;

public class XmlProcessor implements Processor {
    private final AbstractOpenAireProfile profile;
    private final ValidationJob validationJob;
    private long processedRecords;

    private final ValidationIssueRepository validationIssueRepository;
    private final ValidationResultRepository validationResultRepository;

    public XmlProcessor(final AbstractOpenAireProfile profile, final ValidationJob validationJob,
                        final ValidationIssueRepository validationIssueRepository,
                        final ValidationResultRepository validationResultRepository){
        super();
        this.profile = profile;
        this.validationJob = validationJob;
        this.validationIssueRepository = validationIssueRepository;
        this.validationResultRepository = validationResultRepository;
    }


    @Override
    public void process(Exchange exchange) throws Exception {
        String recordXml = exchange.getIn().getBody(String.class);

        final DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        final DocumentBuilder documentBuilder = dbf.newDocumentBuilder();
        Document doc = documentBuilder.parse(new InputSource(new StringReader(recordXml)));
        String recordUrl = extractRecordUrl(doc, "identifier");

        processedRecords++;
        System.out.println("processed " + processedRecords);
        exchange.setProperty("totalRecords", processedRecords);

        if (profile != null) {
            XMLApplicationProfile.ValidationResult validationResult = profile.validate("id", doc); //what id is that?
            System.out.println("DC IDENTIFIER: " + recordUrl);

            Map<String, Guideline.Result> results = validationResult.results();

            ValidationRuleResult validationRuleResult;
            for (Map.Entry entry : results.entrySet()) {
                final Guideline.Result engineResult = (Guideline.Result) entry.getValue();
                String ruleName = entry.getKey().toString();
                validationRuleResult = new ValidationRuleResult();
                constructValidationRuleResult(validationRuleResult, validationJob.id, recordUrl,
                        ruleName, profile, engineResult);
                validationResultRepository.save(validationRuleResult);
                saveValidationIssues(validationJob.id, recordUrl, ruleName, engineResult);

             //   System.out.println(validationRuleResult + " | " + validationRuleResult.hashCode() + "\n");
            }

        }
    }

    private void saveValidationIssues(int validationJobId, String recordUrl, String ruleName, Guideline.Result engineResult) {
        for (String error:engineResult.errors()) {
            /*System.out.println("11111");*/
            ValidationIssue validationIssue = new ValidationIssue();
            validationIssue.validationJobId = validationJobId;
            validationIssue.ruleName = ruleName;
            validationIssue.recordUrl = recordUrl;
            validationIssue.issueType = "ERROR";
            validationIssue.issueText = error;
            validationIssueRepository.save(validationIssue);
/*
            System.out.println(validationIssue);
*/
        }

        for (String warning: engineResult.warnings()){
/*
            System.out.println("22222");
*/
            ValidationIssue validationIssue = new ValidationIssue();
            validationIssue.validationJobId = validationJobId;
            validationIssue.ruleName = ruleName;
            validationIssue.recordUrl = recordUrl;
            validationIssue.issueType = "WARNING";
            validationIssue.issueText = warning;
            validationIssueRepository.save(validationIssue);
/*
            System.out.println(validationIssue);
*/

        }

    }

    /*public void process2(Exchange exchange) throws Exception {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = dbf.newDocumentBuilder();
        String recordXml = exchange.getIn().getBody(String.class);

        // System.out.println("\n\n\n\n\n\n\n\n\n\n\n\n");
        System.out.println(recordXml);
        //   System.out.println("\n\n\n\n\n\n\n\n\n\n\n\n");

        Document doc = db.parse(new InputSource(new StringReader(recordXml)));


        if (profile != null) {
            XMLApplicationProfile.ValidationResult validationResult = profile.validate("id", doc); //what id is that?
            String recordUrl = extractRecordUrl(doc, "identifier");
            System.out.println("DC IDENTIFIER: " + recordUrl);

            Map<String, Guideline.Result> results = validationResult.results();
            for (Map.Entry entry : results.entrySet()) {
                Guideline.Result engineResult = (Guideline.Result) entry.getValue();
                // System.out.println("engine result > " + engineResult);

                String ruleName = entry.getKey().toString();
                //   System.out.println("rulename: " + recordUrl);

                ValidationRuleResult validationRuleResult = new ValidationRuleResult();
                //    System.out.println("validationjob.id " + validationJob.id);
                //    System.out.println("recordUrl " + recordUrl);
                //     System.out.println("rulename " + ruleName);
                //    System.out.println("profile " + profile);
                //    System.out.println("engineResult " + engineResult);



                constructValidationRuleResult(validationRuleResult, validationJob.id, recordUrl,
                        ruleName, profile, engineResult);

                // System.out.println("xaxaxa");
                // System.out.println(validationRuleResult + " | " + validationRuleResult.hashCode() + "\n");
                // resultSum += engineResult.score();

                // elements = nodeListToList(recordNodes); // assign the Elements to the variable



            }
        }

        processedRecords++;
        System.out.println("processed " + processedRecords);
        exchange.setProperty("totalRecords", processedRecords);
    }*/

    //TODO consider throwing exception - UTIL class?
    private String extractRecordUrl(final Document document, final String xmlField) {
        NodeList identifierNodes = document.getElementsByTagName(xmlField);
        if (identifierNodes.getLength() > 0) {
            Element identifierElement = (Element) identifierNodes.item(0);
            return identifierElement.getTextContent();
        }
        return "-";
    }


    private void constructValidationRuleResult(ValidationRuleResult validationRuleResult, int validationJobId, String recordUrl, String ruleName,
                                                               AbstractOpenAireProfile profile, Guideline.Result engineResult) {

        validationRuleResult.validationJobId = validationJobId;
        validationRuleResult.ruleName = ruleName;
        validationRuleResult.ruleWeight = profile.guideline(ruleName).getWeight();
        validationRuleResult.recordUrl = recordUrl;
        validationRuleResult.score = engineResult.score();
        validationRuleResult.status = engineResult.status().toString();
        validationRuleResult.internalError = engineResult.internalError();
    }
}
